package Tie::Directory;
use Data::Dumper;
use strict;
use warnings;


sub new
{
    my $class = shift;
    my %hash  = ();
    tie %hash, $class, @_;
    return \%hash;
}


sub TIEHASH
{
    my $class = shift;
    my $self  = bless { dir => shift }, $class;
    $self->_initialize();
    return $self; 
}


sub _initialize
{
    my $self = shift;
    $self->{dir}                  || die ("\$self->{dir} is not defined.");
    -e $self->{dir}               || die ("$self->{dir} does not seem to exist.");
    -d $self->{dir}               || die ("$self->{dir} does not seem to be a directory.");
}


sub STORE
{
    my $self  = shift;
    my $key   = shift;
    my $value = shift;
    
    $self->{_cached} ||= {};
    $self->{_cached}->{$key} = $value;

    $key = $self->_escape ($key);
    open FP, ">$self->{dir}/$key" || die ("Cannot write-open $self->{dir}/$key");
    print FP Dumper ($value);
    close FP;
}


sub FETCH
{
    my $self  = shift;
    my $key   = shift || return;
    $self->EXISTS ($key) || return;    

    delete $self->{_keys};
    $self->{_cached} ||= {};
    $self->{_cached}->{$key} ||= do {
        my $key = $self->_escape ($key);
        open FP, "<$self->{dir}/$key" || return;
        my $data  = join '', <FP>;
        close FP;
    
        my $VAR1;
        eval "$data";
        $VAR1;
    };

    return $self->{_cached}->{$key};
}


sub _keys
{
    my $self = shift;
    $self->{_keys} ||= do {
        opendir DD, $self->{dir}      || die ("Cannot read-open $self->{dir}");
        my @files = map { $self->_unescape ($_) } sort grep { -f "$self->{dir}/$_" } readdir (DD);
        closedir DD;
        \@files;
    };

    return wantarray ? @{$self->{_keys}} : $self->{_keys};
}


sub FIRSTKEY
{
    my $self  = shift;
    return $self->_keys()->[0];
}


sub NEXTKEY
{
    my $self = shift;
    my $last = shift;
    my @keys = $self->_keys();
    for (my $i=0; $i < @keys; $i++) { $keys[$i] eq $last and return $keys[++$i] }
    return;
}


sub EXISTS
{
    my $self = shift;
    my $key  = shift;
    my $path = $self->{dir} . '/' . $self->_escape ($key);
    return -e $path && -f $path;
}


sub DELETE
{
    my $self = shift;
    my $key  = shift;
    my $path = $self->{dir} . '/' . $self->_escape ($key);
    unlink ($path); 
}


sub CLEAR
{
    my $self = shift;
    for ($self->_keys()) { $self->DELETE ($_) }
}


sub _escape
{
    my $self = shift;
    my $key  = shift;
    $key = "_$key";
    $key =~ s/([^A-Z_-])/"%" . ord($1)/gei;
    return $key;
}


sub _unescape
{
    my $self = shift;
    my $key  = shift;
    $key =~ s/(?:\%(\d+))/chr($1)/eg;
    $key =~ s/^_//;
    return $key;
}


1;
